<?php

namespace Controllers;

use \Zeedhi\Framework\DTO;
use \Zeedhi\Framework\DTO\Response\Error;
use \Zeedhi\Framework\DTO\Response\Message;
use \Zeedhi\Framework\DataSource\DataSet;
use \Zeedhi\Framework\DataSource\FilterCriteria;
use \Zeedhi\Framework\DataSource\Manager;
use \Zeedhi\Framework\DataSource\ParameterBag;
use \Teknisa\GER\Services\CrudService;
use \Doctrine\DBAL\Connection;
use \Teknisa\GER\Utilities\Messages;
// use Teknisa\COC\Controllers;
use Zeedhi\Framework\ORM\DateTime;

class Receita extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $dataSourceName = 'receitam';
    protected $parameterBagColumns = array(
        "CDRECEITA"
    );
    protected $newCode;
    protected $environment;
    protected $parameterBag; 
    protected $connection;
    protected $entityManager;

    public function __construct($crudService, $parameterBag, $dataSourceManager,  $environment, $newCode, WorkflowReceitaController $WorkflowReceitaController, $ValidarReceitasRecursivo, $ValidarReceitasRecursivoVersionamento, $MessagesProcedureInflightor, $receitaService, $entityManager) {
        $this->dataSourceManager                = $dataSourceManager;
        $this->environment                      = $environment;
        $this->newCode                          = $newCode;
        $this->WorkflowReceitaController        = $WorkflowReceitaController;
        $this->ValidarReceitasRecursivo         = $ValidarReceitasRecursivo;
        $this->ValidarReceitasRecursivoVersionamento         = $ValidarReceitasRecursivoVersionamento;
        $this->MessagesProcedureInflightor      = $MessagesProcedureInflightor;
        $this->receitaService                   = $receitaService;
        parent::__construct($crudService, $parameterBag);
        $this->connection                       = $entityManager->getConnection();
        $this->entityManager                    = $entityManager;
    }

    public function save(DTO\Request $request, DTO\Response $response){

        /*
            implementar vericficação de workflow
        */
        $this->dataSourceName = "receitam";
        parent::save($request, $response);

    }
    public function findManual(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = 'manualciam_recsubsemam';
        $this->parameterBagColumns = array("CDRECEITA", "CDFILIAL", "IDVIGENCIA");
        parent::find($request, $response);
    }
    public function findUNidades(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = 'filireceita_filial_receita';
        $this->parameterBagColumns = array("CDRECEITA");
        parent::find($request, $response);
    }
    public function getDadoswFlow(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = 'workflow_dadoswflow';
        $this->parameterBagColumns = array("DSPRIMARYKEY", "NMTABELAWFLOW", "NRORG");
        parent::find($request, $response);
    }

    //Validates the 'Receitas' situation change linked to Menu Grid
    public function validateChangeOfSituation(DTO\Request\Row $request, DTO\Response $response, $delete  = null){
        try{
            
            $P_MENU_EXPIRADO = $delete ? "S" : "N";
            $row = $request->getRow();
            $arrayErrosMsg = array();
            //Valida utilização da Receita na estrutura de Menu Grid antiga
            $resultado = $this->ValidarReceitasRecursivo->executeProcedure($this->environment->getCdFilial(), $row['CDRECEITA'], $row['P_IDVIGENCIA'], $this->environment->getNrOrgTrab(), $this->environment->getCdOperador(), $P_MENU_EXPIRADO);  
            if($resultado['P_NRLISTAERROS'] !== null) {
                $msgsError = $this->MessagesProcedureInflightor->concatErrorMessages($this->environment->getCdFilial(), $resultado['P_NRLISTAERROS'], $response, $row['DSRECEITA'], false);
                $arrayErrosMsg = array_merge($arrayErrosMsg, $msgsError);
            }
            //Valida utilização da Receita na nova estrutura de Menu Grid - Versionamento 18/01/2018
            /* REMOVIDA POR DESCONTINUIDADE DO MGD
            $resultado = array();
            
            $resultado = $this->ValidarReceitasRecursivoVersionamento->executeProcedure($this->environment->getCdFilial(), $row['CDRECEITA'], $row['P_IDVIGENCIA'], $this->environment->getNrOrgTrab(), $this->environment->getCdOperador(), $P_MENU_EXPIRADO);  
            if($resultado['P_NRLISTAERROS'] !== null) {
                $msgsError = $this->MessagesProcedureInflightor->concatErrorMessages($this->environment->getCdFilial(), $resultado['P_NRLISTAERROS'], $response, $row['DSRECEITA'], false);
                $arrayErrosMsg = array_merge($arrayErrosMsg, $msgsError);
            }
            */
            //Valida utilização da Receita nas demais refêrencias à tabela de Receitam
            $resultado = array();
            $resultado = $this->receitaService->findScopeReferenciaReceita($row);
            $erroMsg   = array();
            if(!empty($resultado)){
                $erroMsg = $this->receitaService->convertMessage($resultado);
                $arrayErrosMsg = array_merge($arrayErrosMsg, $erroMsg);
            }
            if($delete){
                return $arrayErrosMsg;
            } else {
                return $response->addDataSet(new DataSet("ListaErros", $arrayErrosMsg));
            }
        } catch (\Exception $e){
            $response->setError(new Error("Erro ao validar edição da Receita ". $e->getMessage(), 500));
        }
    }

    public function deleteReceita(DTO\Request\Row $request, DTO\Response $response){
        $connection = $this->connection;
        try {

            $connection->beginTransaction();

            $row = $request->getRow();
            $validation = $this->validateChangeOfSituation($request, $response, true);
            if(empty($validation)){
                //Deleta todas as Dependências da Tabela de Receitam
                $callBackReferencias = $this->receitaService->deleteScopeReceita($row);
                //Exclui a Receita
                $request = $this->crudService->buildDataSetIfRow($request, $this->dataSourceName);
                $deleteDataset = $request->getDataSet();
                $this->crudService->delete($response, $deleteDataset);
                $connection->commit();
            } else {
                $connection->rollback();
                //Retorna Lista de Erros 
                return $response->addDataSet(new DataSet("ListaErros", $validation));
            }
        } catch (\Exception $e){
            $connection->rollback();
            $response->setError(new Error("Erro no processo de Exclusão da Receita ". $e->getMessage(), 500));
        }
    }

    private function findReceitaMenuGrid ($row){
        $condition = array(
            array(
                "columnName" => "CDRECEITA",
                "operator" => "=",
                "value" => $row["CDRECEITA"]
            )
        );
        $filterCriteria = new FilterCriteria('manualciam_recsubsemam', $condition);
        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
        $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
        if(empty($dataSet->getRows()))
            return [];
        return $dataSet->getRows();
    }

    public function salvaNovaReceita(DTO\Request\Row $request, DTO\Response $response){
        /*
            Implementando processo de verificação de Workflow
        */

        $this->dataSourceName = "receitam";
        $row = $request->getRow();
        $currentTime = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");

        if($row['__is_new']){

            // $row['QTRENDIMENTOREC'] = floatval(str_replace(',', '.', $row['QTRENDIMENTOREC']));
            // $row['CDRECEITA'] =  $this->newCode->getCodeByTableName('RECEITAM', $this->environment->getNrOrgTrab());
            $row['DTALTSTATUS'] =  $currentTime;
            $row['IDRECVATIV'] = 'S';
            // $row["__columns_to_return"] = ["CDRECEITA"];
        }
        parent::save($request, $response);
    }
    private function updateReceita($row, $response){
        $this->dataSourceName = 'receitam';
        $newRow = $row;
        $newRow["IDANALIZEATIV"] = "P";
        parent::save($this->createRequest(array($newRow), $this->dataSourceName), $response);
    }

    public function returnColumns(){
        return $this->crudService->getColumnsToReturn();
    }
    private function createRequest($row, $dataSourceName){
        $newDataSet = new DataSet($dataSourceName, $row);
        $newRequest = new DTO\Request\DataSet($newDataSet, "", "", "");
        return $newRequest;
    }
    private function getVinculosModulos($row, $TABELA){
        $params = array(
            "CDRECEITA" => $row['CDRECEITA'],
            "NRORG" => $row['NRORG']
        );
        $query = "SELECT COUNT(*) AS EXISTE FROM $TABELA WHERE CDRECEITA = :CDRECEITA AND NRORG = :NRORG";
        $rows = $this->connection->fetchAll($query, $params);
        return $rows;
    }

    private function getArrayVinculos($row){
        $array_vinculos = array(
            'PRECOSESUBGRU' => $this->getVinculosModulos($row, 'PRECOSESUBGRU'),
            'VARIACAOREC' => $this->getVinculosModulos($row, 'VARIACAOREC'),
            'CATEGORIAREC' => $this->getVinculosModulos($row, 'CATEGORIAREC'),
            'RECSUBSEMAM' => $this->getVinculosModulos($row, 'RECSUBSEMAM'),
            'FLUXPRODREC' => $this->getVinculosModulos($row, 'FLUXPRODREC'),
            'PADRAOPRODREC' => $this->getVinculosModulos($row, 'PADRAOPRODREC')
        );
        return $array_vinculos;
    }

    public function generateWflow(DTO\Request\Row $request, DTO\Response $response){
        //NRPROCESSOAPROVA = '1' -> Processo Aprovação Receita fixed
        //NRITEMPROCAPROVA = '1' -> Inclusão Receita
        try {
            $row = $request->getRow();
            if($row['NRITEMPROCAPROVA']==2){
                //Valida filhos da receita para solicitar a exclusao
                $array_vinculos = $this->getArrayVinculos($row);
                $flag_vinculos = false;
                foreach ($array_vinculos as $vinculos) {
                    if($vinculos[0]['EXISTE']>0)
                        $flag_vinculos = true;
                }
            }
            if($flag_vinculos){
                $response->addDataSet(new DataSet('vinculos', $array_vinculos));
                $response->setStatus(DTO\Response::STATUS_ERROR);
            } else {
                //Recebe todos os operadores que deverão aprovar ou não o processo em questão
                $operEnvolvidos = $this->WorkflowReceitaController->getOperatorsInvolved($row, $response);
                if(!empty($operEnvolvidos)){
                    //Altera campo de controle de WorkFlow na Receitam - IDANALIZEATIV
                    $this->updateReceita($row, $response);
                    //Gerar Processo de WorkFlow com o Operador logado
                    $workflowGerado = $this->WorkflowReceitaController->newProcessWorkflow($row, $request, $response);
                    //Cria solicitação para todos os Operadores Responsáveis
                    $this->WorkflowReceitaController->operWorkflow($operEnvolvidos, $workflowGerado, $response);
                    //Solicita alteração na tabela de RECEITAM para ativar a receita
                    $row["NMTABELAWFLOW"] = 'RECEITAM';
                    $this->WorkflowReceitaController->saveDadoswflow($row, $workflowGerado, $response);
                } else {
                    $response->setError(new Error("Error in WORKFLOW process. Nenhum operador encontrado para o processo de 'Inclusão Receita'" ,500));
                }
            }
        }catch (\Exception $e){
            $response->setError(new Error("Error in WORKFLOW process. ". $e->getMessage(), 500));
        }

    }
    public function generateWflowEdit(DTO\Request $request, DTO\Response $response){
        try{

            //IDTIPOPERWFLOW -- 1 -> UPDATE
            //               -- 2 -> INSERT
            //               -- 3 -> DELETE
            $row = $request->getDataSet()->getRows();
            $oldRow = $row[0];
            $validateProcess = $row[1];
            $newRow = $row[2];
            //Progress
            $oldRow['NRPROCESSOAPROVA'] = 1;
            foreach ($validateProcess as $field) {
                if(!empty($field)){
                    $oldRow['NRITEMPROCAPROVA'] = $field[array_keys($field)[0]];
                    $operator = $this->WorkflowReceitaController->getOperatorsInvolved($oldRow, $response);
                    if(!empty($operator)) {
                        //Gerar Processo de WorkFlow com o Operador logado
                        $oldRow['IDTIPOPERWFLOW'] = 1;
                        $oldRow['DSMOTIVOACAO'] = $newRow['DSMOTIVOACAO'];
                        $oldRow['IDSITWORKFLOW'] = 1;
                        //Altera campo de controle de WorkFlow na Receitam - IDANALIZEATIV
                        $this->updateReceita($oldRow, $response);
                        $workflowGerado = $this->WorkflowReceitaController->newProcessWorkflow($oldRow, $request, $response);
                        //Cria solicitação para todos os Operadores Responsáveis
                        $this->WorkflowReceitaController->operWorkflow($operator, $workflowGerado, $response);
                        //Solicita alteração na tabela de RECEITAM para as alterações feitas
                        $newRow['NMCAMPOWFLOW'] = array_keys($field)[0];
                        $oldRow['DSINFORANTERIOR'] = $oldRow[array_keys($field)[0]];
                        $newRow['DSINFORATUAL'] = $newRow[array_keys($field)[0]];
                        $newRow["NMTABELAWFLOW"] = 'RECEITAM';
                        $this->WorkflowReceitaController->updateDadoswflow($oldRow, $newRow, $workflowGerado, $response);
                    }else {
                        $itemprocaprova = $this->WorkflowReceitaController->getItemprocaprova($oldRow, $response);
                        $response->setError(new Error("Error in WORKFLOW process. Nenhum operador encontrado para o processo de '".$itemprocaprova[0]['NMITEMPROCAPROVA']."'" ,500));
                    }
                }
            }
        } catch (\Exception $e){
            $response->setError(new Error("Error in WORKFLOW process. ". $e->getMessage(), 500));
        }
    }
    
    public function removePreferenciaComposicao(DTO\Request\DataSet $request, DTO\Response $response){
        
        $dataset = $request->getDataSet()->getRows();
        $datasetprodutos = array();
        $datasetSubreceitas = array();
        for ($i = 0; $i < sizeof($dataset); $i++) {
            if($dataset[$i]['ID'] == 'P'){
                $dataset[$i]['CDPRODUTO'] = $dataset[$i]['CODIGO'];
                array_push($datasetprodutos, $dataset[$i]);
            }
            if($dataset[$i]['ID'] == 'S'){
                $dataset[$i]['CDSUBRECEITA'] = $dataset[$i]['CODIGO'];
                array_push($datasetSubreceitas, $dataset[$i]);
            }
        }
        
        $this->salvaProdutosComposicao($datasetprodutos);
        $this->salvaSubreceitasComposicao($datasetSubreceitas);
        $response->addMessage(new Message('Preferência(s) alterada(s).'));
        
    }
    
    public function salvaPreferenciaComposicao(DTO\Request\DataSet $request, DTO\Response $response){
        
        $this->validaPreferenciaComposicaoESalva($request, $response);
        
    }
    
    private function numeroOpcoesRestantes($cdreceita, $nrvariacaorec){
        
        $preferenciasJaExistentes = $this->preferenciasJaExistentes($cdreceita, $nrvariacaorec);
        $numeroOpcoes = $this->numeroOpcoes($cdreceita, $nrvariacaorec);
        return $numeroOpcoes - $preferenciasJaExistentes;
        
    }
    
    private function numeroOpcoes($cdreceita, $nrvariacaorec){

		$filter = array(
			array(
				"columnName" => "CDRECEITA",
				"operator" => "=",
				"value" => $cdreceita
			),
			array(
				"columnName" => "NRVARIACAOREC",
				"operator" => "=",
				"value" => $nrvariacaorec
			)
		);
		
		$filterCriteria = new FilterCriteria('variacaorec', $filter);
		$filterCriteria =  $this->crudService->populateParameterBag($filterCriteria, array());
		$dataSet = $this->dataSourceManager->findBy($filterCriteria);
		$registros = $dataSet->getRows();
		return $registros[0]['NROPCOES'];
        
    }
    
    private function preferenciasJaExistentes($cdreceita, $nrvariacaorec){

		$filter = array(
			array(
				"columnName" => "CDRECEITA",
				"operator" => "=",
				"value" => $cdreceita
			),
			array(
				"columnName" => "NRVARIACAOREC",
				"operator" => "=",
				"value" => $nrvariacaorec
			)
		);
		
		$filterCriteria = new FilterCriteria('coc01400_receitas_produtos', $filter);
		$filterCriteria =  $this->crudService->populateParameterBag($filterCriteria, array('NRORG', 'CDRECEITA', 'NRVARIACAOREC'));
		$dataSet = $this->dataSourceManager->findBy($filterCriteria);
		$registros = $dataSet->getRows();
		
		$preferenciasExistentes = 0;
		
		for ($i = 0; $i < sizeof($registros); $i++) {
		     if($registros[$i]['IDPREFERENCIA'] === 'S')
		        $preferenciasExistentes += 1;        
		}
		
		return $preferenciasExistentes;
        
    }
    
    private function validaPreferenciaComposicaoESalva($request, $response){
        
        $dataset = $request->getDataSet()->getRows();
        
        $cdreceita = $dataset[0]['CDRECEITA'];
        $nrvariacaorec = $dataset[0]['NRVARIACAOREC'];
        $numeroOpcoesRestantes = $this->numeroOpcoesRestantes($cdreceita, $nrvariacaorec);
        $numeroAlteracoes = 0;
        for ($i = 0; $i < sizeof($dataset); $i++) {
            if($dataset[$i]['IDPREFERENCIA'] != $dataset[$i]['IDPREFERENCIA_OLD'])
                $numeroAlteracoes += 1;
        }
        
        if($numeroAlteracoes > $numeroOpcoesRestantes){
            $response->setError(new Error("Operação bloqueada. O máximo de preferências é ". $this->numeroOpcoes($cdreceita, $nrvariacaorec).'.', 500));
        }else {
            $datasetprodutos = array();
            $datasetSubreceitas = array();
            for ($i = 0; $i < sizeof($dataset); $i++) {
                if($dataset[$i]['ID'] == 'P'){
                    $dataset[$i]['CDPRODUTO'] = $dataset[$i]['CODIGO'];
                    array_push($datasetprodutos, $dataset[$i]);
                }
                if($dataset[$i]['ID'] == 'S'){
                    $dataset[$i]['CDSUBRECEITA'] = $dataset[$i]['CODIGO'];
                    array_push($datasetSubreceitas, $dataset[$i]);
                }
            }
            
            $this->salvaProdutosComposicao($datasetprodutos);
            $this->salvaSubreceitasComposicao($datasetSubreceitas);
            $response->addMessage(new Message('Preferência(s) alterada(s).'));
            
        }
        
    }
    
    private function salvaProdutosComposicao($datasetprodutos){
        
        $novoDataset = new DataSet('proreceitam', $datasetprodutos);
		$novoRequest = new DTO\Request\DataSet($novoDataset, '', '', '');
		$produtosParaSalvar = $this->crudService->getRowsToSave($novoRequest, "proreceitam");
		$dataSetToSave = new DataSet('proreceitam', $produtosParaSalvar);
		$this->dataSourceManager->persist($dataSetToSave);
        
    }
    
    private function salvaSubreceitasComposicao($datasetSubreceitas){
        
        $novoDataset = new DataSet('subreceitam', $datasetSubreceitas);
		$novoRequest = new DTO\Request\DataSet($novoDataset, '', '', '');
		$SubreceitasParaSalvar = $this->crudService->getRowsToSave($novoRequest, "subreceitam");
		$dataSetToSave = new DataSet('subreceitam', $SubreceitasParaSalvar);
		$this->dataSourceManager->persist($dataSetToSave);
        
    }
    
}