<?php
namespace Controllers;

use Zeedhi\Framework\Report\ReportService;
use \Zeedhi\Framework\DTO;
use \Zeedhi\Framework\DataSource\FilterCriteria;
use \Zeedhi\Framework\DTO\Request\Row;

class RelatoriodoManual extends \Zeedhi\Framework\Controller\Report {

    protected $birtStrategy;

    public function __construct(ReportService $reportService, $birtStrategy, $environment) {
        $this->birtStrategy = $birtStrategy;
        parent::__construct($reportService);
        $this->environment = $environment;
    }

    protected $reportMapping = array (
        'COC03602' => array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "CDFILIAL"         =>"P_CDFILIAL",
              "NRORG"            =>"P_NRORG",
              "CDMANCIA"         =>"P_CDMANCIA",
              "CDCIA"            =>"P_CDCIA",
              "SUBRECEITASMENU"  =>"P_SUBRECEITAS"
            )
        ),
        'COC03603' => array ( //Impressão do Menu Grid
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "NRORG"         =>"NRORG",
              "CDFILIAL"      =>"CDFILIAL",
              "CDMANCIA"      =>"CDMANCIA",
              "CDCIA"         =>"CDCIA",
              "CDCLASSE"      =>"CDCLASSE",
              "SERVICOCLASSE" =>"NRSERVICOCIAFILI",
              "SERVCLASSE"    =>"SERVCLASSE",
              "SUBGRU"        =>"CDSUBGRU",
              "SUBGRUPO"      =>"SUBGRUPO",
              "CICLO"         =>"CDCICLO",
              "FILESERVPATH"  =>"FILESERVPATH",
              "HIDEIMG"       =>"HIDEIMG",
              "ONLYVIGSBG"    =>"ONLYVIGSBG"
            )
        ),
        'COC03604' => array ( //Impressão do Menu Grid - Itens Alterados
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "NRORG"         =>"NRORG",
              "CDFILIAL"      =>"CDFILIAL",
              "PARACLIENTE"   =>"PARACLIENTE",
              "CDMANCIA"      =>"CDMANCIA",
              "CDCIA"         =>"CDCIA",
              "CDCLASSE"      =>"CDCLASSE",
              "SERVICOCLASSE" =>"NRSERVICOCIAFILI",
              "SERVCLASSE"    =>"SERVCLASSE",
              "SUBGRU"        =>"CDSUBGRU",
              "SUBGRUPO"      =>"SUBGRUPO",
              "CICLO"         =>"CDCICLO",
              "FILESERVPATH"  =>"FILESERVPATH",
              "HIDEIMG"       =>"HIDEIMG"
            )
        ),
        'COCB3602' => array ( //Produtos / Receitas Inativos
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "NRORG"            =>"NRORG",
              "CDFILIAL"         =>"CDFILIAL",
              "CDCIA"            =>"CDCIA",
              "CDMANCIA"         =>"CDMANCIA",
              "ONLYVIGSBG"       =>"ONLYVIGSBG"
            )
        ),
        'COCC3602' => array ( //Alteração de Menu Grid
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "NRORG"          =>"NRORG",
              "IDTIPOPERWFLOW" =>"P_IDTIPOPERWFLOW",
              "CDFILIAL"         =>"CDFILIAL"
            )
        ),
        'COCD3602' => array ( // Menu Grid -Vigência
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "NRORG"          =>"NRORG",
              "CDFILIAL" =>"P_CDFILIAL",
              "P_CDCIA" =>"P_CDCIA",
              "IDSITUAPROPCIA" =>"P_VIGENCIA"
            )
        ),
        'COC03614' => array ( // Pendências de Vínculo
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "CDFILIAL" =>"CDFILIAL",
              "CIA" =>"CIA",
              "MANUAL" =>"MANUAL",
              "NRORG" =>"NRORG"
            )
        )
    );

    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
      $row = $request->getRow();
      if($row['REPORTNAME'] === 'COCC3602') {
          $row['CDFILIAL'] = $this->environment->getCdFilial();
      }
      if (isset($row['__locale']) && $row['__locale']){
        $this->birtStrategy->setReportLocale($row['__locale']);
      }
      if (isset($row['__format']) && $row['__format']){
        $this->birtStrategy->setReportFormat($row['__format']);
      }
    }

}
